package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.*;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.DecentralizedRequestMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.DecentralizedRequestVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * (分散采购申请)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (分散采购申请) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class DecentralizedRequestServiceImpl extends ServiceImpl<DecentralizedRequestMapper, DecentralizedRequest> implements DecentralizedRequestService {

    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private ComplianceCheckService complianceCheckService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private PurchaseListService purchaseListService;

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;
    @Autowired

    private OssFile ossFile;
    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Override//分散采购申请分页
    public IPage<DecentralizedRequestVO> getDecentralizedRequestVOPage
            (@Param("page") IPage<DecentralizedRequest> page, @Param(Constants.WRAPPER) QueryWrapper<DecentralizedRequest> qw) {

        IPage<DecentralizedRequestVO> decentralizedRequestVOPage = baseMapper.getDecentralizedRequestVOPage(page, qw);

        return decentralizedRequestVOPage;
    }


    @Override//分散采购申请列表
    public List<DecentralizedRequestVO> getDecentralizedRequestVOList(@Param(Constants.WRAPPER) QueryWrapper<DecentralizedRequest> qw) {

        List<DecentralizedRequestVO> decentralizedRequestVOList = baseMapper.getDecentralizedRequestVOList(qw);

        return decentralizedRequestVOList;
    }

    @Override//通过ID查询分散采购申请
    public DecentralizedRequestVO getRequestById(String decentralizedRequestId) {

        DecentralizedRequestVO decentralizedRequestVO = baseMapper.getDecentralizedRequest(decentralizedRequestId);

        List<DecentralizeDetails> decentralizeDetailsList = decentralizeDetailsService.getDecentralizeDetailsList(decentralizedRequestId);

        decentralizedRequestVO.setDecentralizeDetailsVOList(decentralizeDetailsList);

        return decentralizedRequestVO;
    }

    @Override//创建分散采购申请
    public DecentralizedRequest addRequest(List<DecentralizedRequestDTO> decentralizedRequestDTOList, DLPUser dlpUser) {

        DecentralizedRequest decentralizedRequest = new DecentralizedRequest();

        decentralizedRequest.setId(IdWorker.get32UUID().toUpperCase());

        decentralizedRequest.setApplicantId(dlpUser.getId());

        decentralizedRequest.setStatus(0);

        List<DecentralizeDetails> list = new ArrayList<>();
        //接收分散采购DTO，将其赋值给分散采购明细
        for (DecentralizedRequestDTO decentralizedRequestDTO : decentralizedRequestDTOList) {
            //新添加试剂耗材

            DecentralizeDetails decentralizedDetails = new DecentralizeDetails();

            BeanUtils.copyProperties(decentralizedRequestDTO, decentralizedDetails);

            decentralizedDetails.setId(IdWorker.get32UUID().toUpperCase());

            decentralizedDetails.setDecentralizedRequestId(decentralizedRequest.getId());

            list.add(decentralizedDetails);
        }
        if (this.save(decentralizedRequest) & decentralizeDetailsService.saveBatch(list)) {
            return decentralizedRequest;
        } else return null;
    }

    @Override
    public DecentralizeDetails addDetails(DecentralizedRequestDTO decentralizedRequestDTO) {

        DecentralizeDetails decentralizeDetails = new DecentralizeDetails();

        BeanUtils.copyProperties(decentralizedRequestDTO, decentralizeDetails);

        if (decentralizeDetailsService.save(decentralizeDetails)) {
            return decentralizeDetails;
        } else throw new RuntimeException(String.format("保存失败"));
    }

    @Override//修改分散采购申请明细
    public DecentralizeDetails editRequest(DecentralizedRequestDTO decentralizedRequestDTO) {

        DecentralizeDetails decentralizeDetails = decentralizeDetailsService.getById(decentralizedRequestDTO.getDecentralizeDetailsId());

        BeanUtils.copyProperties(decentralizedRequestDTO, decentralizeDetails);

        if (decentralizeDetailsService.updateById(decentralizeDetails)) {
            return decentralizeDetails;
        } else return null;

    }

    @Override//删除分散采购申请
    public Boolean delRequestById(String decentralizedRequestId) {

        LambdaQueryWrapper<DecentralizeDetails> decentralizeDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        decentralizeDetailsLambdaQueryWrapper.eq(DecentralizeDetails::getDecentralizedRequestId, decentralizedRequestId);

        List<DecentralizeDetails> list = decentralizeDetailsService.list(decentralizeDetailsLambdaQueryWrapper);

        if (list != null) {
            decentralizeDetailsService.removeBatchByIds(list);
        }

        return this.removeById(this.getById(decentralizedRequestId));

    }

    @Override//提交分散采购申请
    public DecentralizedRequest commitRequest(List<DecentralizedRequestDTO> decentralizedRequestDTOList, DLPUser dlpUser) {
        String decentralizedRequestId = decentralizedRequestDTOList.get(0).getDecentralizedRequestId();

        if (decentralizedRequestId == null) {

            DecentralizedRequest decentralizedRequest = this.addRequest(decentralizedRequestDTOList, dlpUser);

            decentralizedRequest.setStatus(1);

            decentralizedRequest.setCommitTime(LocalDateTime.now());
            decentralizedRequest.setDateOfApplication(LocalDateTime.now());

            if (this.updateById(decentralizedRequest)) {
                return decentralizedRequest;
            } else throw new RuntimeException(String.format("提交失败"));
        } else {

            DecentralizedRequest byId = this.getById(decentralizedRequestId);

            byId.setStatus(1);

            byId.setCommitTime(LocalDateTime.now());

            if (this.updateById(byId)) {
                return byId;
            } else throw new RuntimeException(String.format("提交失败"));
        }

    }

    @Override//一级审核
    public DecentralizedRequest primaryAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = this.getById(auditAndApproveDto.getUuId());

        if (byId.getStatus() != 1) {
            throw new RuntimeException(String.format("当前状态不能审核"));
        }

        byId.setPrimaryAuditorId(dlpUser.getId());

        byId.setAuditResultOfPrimary(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfPrimary(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfPrimary(LocalDateTime.now());

        if (auditAndApproveDto.getAuditResult() == true) {
            byId.setStatus(2);
        } else {
            byId.setStatus(-2);
        }

        if (this.updateById(byId)) {
            return byId;
        } else return null;


    }

    @Override//二级审核
    public DecentralizedRequest secondaryAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {

        DecentralizedRequest byId = this.getById(auditAndApproveDto.getUuId());

        if (byId.getStatus() != 2) {
            throw new RuntimeException(String.format("当前状态不能审核"));
        }

        byId.setSecondaryAuditorId(dlpUser.getId());

        byId.setAuditResultOfSecondary(auditAndApproveDto.getAuditResult());

        byId.setAuditOpinionOfSecondary(auditAndApproveDto.getAuditOpinion());

        byId.setAuditTimeOfSecondary(LocalDateTime.now());

        if (auditAndApproveDto.getAuditResult() == true) {
            byId.setStatus(3);
        } else {
            byId.setStatus(-3);
        }

        if (this.updateById(byId)) {
            return byId;
        } else return null;
    }

    @Transactional
    @Override//三级审核
    public DecentralizedRequest threeLevelAuditRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDTO) {

        DecentralizedRequest byId = this.getById(auditAndApproveDTO.getUuId());

        if (byId.getStatus() != 3) {
            throw new RuntimeException(String.format("当前状态不能审核"));
        }

        byId.setThreeLevelAuditId(dlpUser.getId());

        byId.setAuditResultOfThreeLevel(auditAndApproveDTO.getAuditResult());

        byId.setAuditOpinionOfThreeLevel(auditAndApproveDTO.getAuditOpinion());

        byId.setAuditTimeOfThreeLevel(LocalDateTime.now());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(4);
        } else {

            byId.setStatus(-4);
        }
        if (this.updateById(byId)) {
            return byId;
        } else return null;
    }

    @Override//审批
    @Transactional
    public DecentralizedRequest approveRequest(DLPUser dlpUser, AuditAndApproveDTO auditAndApproveDto) {
        DecentralizedRequest byId = this.getById(auditAndApproveDto.getUuId());
        if (byId.getStatus() != 4) {
            throw new RuntimeException(String.format("当前状态不能审批"));
        }
        byId.setApproverId(dlpUser.getId());
        byId.setResultOfApproval(auditAndApproveDto.getApproveResult());
        byId.setOpinionOfApproval(auditAndApproveDto.getApproveOpinion());
        byId.setApprovalOfTime(LocalDateTime.now());
        if (auditAndApproveDto.getApproveResult() == true) {
            List<DecentralizeDetails> decentralizeDetailsList = decentralizeDetailsService.list(new LambdaQueryWrapper<DecentralizeDetails>().eq(DecentralizeDetails::getDecentralizedRequestId, byId.getId()));
            //创建采购清单
            List<PurchaseListDetails> purchaseListDetailsList = purchaseListService.addList(byId.getId(), 0);

            if (purchaseListDetailsList == null) {
                throw new RuntimeException(String.format("出错了，请及时联系工作人员"));
            }

            byId.setStatus(5);
        } else {
            byId.setStatus(-5);
        }
        if (this.updateById(byId)) {
            return byId;
        } else return null;
    }

    @Override
    public void purchaseRequestTablePDF(String id, String type, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("purchaseRequestTablePDF.................");

        int x = 1;

        double total = 0;


        String firstAuditName = null;
        String secondAuditName = null;
        String thirdAuditName = null;
        String fourthAuditName = null;
        String commitTime = null;

        //-----------------------------
        // 生成 word 版本的 采购目录

        DecentralizedRequestVO requestById = this.getRequestById(id);

        List<DecentralizeDetails> decentralizeDetailsList = requestById.getDecentralizeDetailsVOList();

        if (decentralizeDetailsList == null) {

            throw new RuntimeException(String.format("未查询到相关信息"));
        }

        List<DecentralizeDetails> voList = new ArrayList<>();

        for (DecentralizeDetails decentralizeDetails : decentralizeDetailsList) {

            if (!decentralizeDetails.getCategory().equals("标准物质")) {
                voList.add(decentralizeDetails);
            }
        }

        firstAuditName = requestById.getPrimaryAuditorName();
        secondAuditName = requestById.getSecondaryAuditorName();
        thirdAuditName = requestById.getThreeLevelAuditName();
        fourthAuditName = requestById.getApproverName();
        commitTime = LocalDateTimeUtil.format(requestById.getCommitTime(), "yyyy年MM月dd日");


        String applyFileName = "试剂耗材分散采购申请表-" + id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "试剂耗材分散采购申请表.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);
        builder.bind("voList", policy).

                build();

        /**
         * 循环表格
         */
        double finalTotal = total;
        String finalFourthAuditName = fourthAuditName;
        String finalSecondAuditName = secondAuditName;
        String finalThirdAuditName = thirdAuditName;
        String finalFirstAuditName = firstAuditName;
        String finalCommitTime = commitTime;
        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(


                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("finalFirstAuditName", finalFirstAuditName);
                    put("finalSecondAuditName", finalSecondAuditName);
                    put("finalThirdAuditName", finalThirdAuditName);
                    put("finalFourthAuditName", finalFourthAuditName);
                    put("applicantName", requestById.getApplicantName());
                    put("commitTme", LocalDateTimeUtil.format(requestById.getCommitTime(), "yyyy年MM月dd日"));
                    put("auditTimeOfPrimary", LocalDateTimeUtil.format(requestById.getAuditTimeOfPrimary(), "yyyy年MM月dd日"));
                    put("auditTimeOfSecondary", LocalDateTimeUtil.format(requestById.getAuditTimeOfSecondary(), "yyyy年MM月dd日"));
                    put("auditTimeOfThreeLeve", LocalDateTimeUtil.format(requestById.getAuditTimeOfThreeLevel(), "yyyy年MM月dd日"));
                    put("approvalOfTime", LocalDateTimeUtil.format(requestById.getApprovalOfTime(), "yyyy年MM月dd日"));
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().

                asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "DecentralizeRequest" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }
}




