package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.entity.CheckSchedule;
import digital.laboratory.platform.reagent.entity.PeriodVerificationPlan;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.mapper.PeriodVerificationPlanMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.CheckScheduleVO;
import digital.laboratory.platform.reagent.vo.PeriodVerificationPlanVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * (标准物质期间核查计划和确认表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查计划和确认表) 服务实现类
 */
@Service
public class PeriodVerificationPlanServiceImpl extends ServiceImpl<PeriodVerificationPlanMapper, PeriodVerificationPlan> implements PeriodVerificationPlanService {

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private PeriodVerificationImplementationService periodVerificationImplementationService;

    @Autowired
    private CheckScheduleService checkScheduleService;

    @Override
    public List<PeriodVerificationPlanVO> getPeriodVerificationPlanVOList(String checkScheduleId) {

        List<PeriodVerificationPlanVO> periodVerificationPlanVOPage = baseMapper.getPeriodVerificationPlanVOList(checkScheduleId);

        for (PeriodVerificationPlanVO periodVerificationPlanVO : periodVerificationPlanVOPage) {
            ReferenceMaterial referenceMaterial = referenceMaterialService.getById(periodVerificationPlanVO.getReferenceMaterialId());
            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());
            periodVerificationPlanVO.setReferenceMaterialName(reagentConsumables.getReagentConsumableName());
            periodVerificationPlanVO.setDeviationAndUncertainty(reagentConsumables.getDeviationOrUncertainty());
            periodVerificationPlanVO.setStandardValueOrPurity(reagentConsumables.getStandardValueOrPurity());
        }
        return periodVerificationPlanVOPage;
    }

    @Override
    public IPage<PeriodVerificationPlanVO> getPeriodVerificationPlanVOPage(Page<PeriodVerificationPlan> page, QueryWrapper<PeriodVerificationPlan> qw) {

        IPage<PeriodVerificationPlanVO> periodVerificationPlanVOPage = baseMapper.getPeriodVerificationPlanVOPage(page, qw);

        return periodVerificationPlanVOPage;
    }

    @Override
    public void getPlan() {

        ArrayList<PeriodVerificationPlanVO> periodVerificationPlanVOArrayList = new ArrayList<>();
        //查询审核已通过的期间核查计划
        List<CheckSchedule> checkScheduleList = checkScheduleService.list(Wrappers.<CheckSchedule>query().eq("status", 6));
        //获取所有的计划明细
        for (CheckSchedule checkSchedule : checkScheduleList) {

            CheckScheduleVO checkScheduleVO = checkScheduleService.getCheckScheduleVO(checkSchedule.getId());

            List<PeriodVerificationPlanVO> periodVerificationPlanVOS = checkScheduleVO.getPeriodVerificationPlanVOS();

            periodVerificationPlanVOArrayList.addAll(periodVerificationPlanVOS);
        }
        //循环计划明细，判断是否需要创建该期间核查计划
        /*
         * 每一次创建计划之后，不会存在下次核查日期，在核查人未核查之前不会创建新的
         * 若核查人核查完成之后，就会生成下次核查日期，也就会进入循环
         * 若条件满足，则创建期间核查，同时再次清空下次核查日期，也就是在新创建，但未核查的情况下，并不会重复创建*/
        for (PeriodVerificationPlan periodVerificationPlan : periodVerificationPlanVOArrayList) {

            ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(periodVerificationPlan.getReferenceMaterialId());
            //判断标准物质是否正常使用
            if (referenceMaterialServiceById.getStatus() != -1 & referenceMaterialServiceById.getStatus() != -4 & periodVerificationPlan.getDateOfNextCheck() != null) {

                if (periodVerificationPlan.getDateOfNextCheck().isBefore(LocalDate.now().plusDays(3L))) {

                    periodVerificationImplementationService.addById(periodVerificationPlan);

                    periodVerificationPlan.setDateOfNextCheck(null);

                    this.updateById(periodVerificationPlan);

                }
            }
        }
    }
}
