package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.PurchaseListMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * (采购清单)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class PurchaseListServiceImpl extends ServiceImpl<PurchaseListMapper, PurchaseList> implements PurchaseListService {

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private WarehousingRecordFormService warehousingRecordFormService;

    @Autowired
    private CatalogueDetailsService catalogueDetailsService;

    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private SupplierInformationService supplierInformationService;
    @Autowired
    private DecentralizedRequestService decentralizedRequestService;
    @Autowired
    private PurchasingPlanService purchasingPlanService;

    @Autowired
    private ProcurementContentService procurementContentService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private CentralizedRequestService centralizedRequestService;

    @Autowired
    private DetailsOfCentralizedService detailsOfCentralizedService;
    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;
    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private BatchDetailsService batchDetailsService;

    @Override//通过ID查找清单
    public PurchaseListVO getPurchaseList(String purchaseListId) {

        PurchaseListVO purchaseListVO = new PurchaseListVO();//采购清单VO

        PurchaseList purchaseList = this.getById(purchaseListId);
        List<PurchaseListDetailsVO> purchaseListDetailsVOList = purchaseListDetailsService.getPurchaseListDetailsVOList(purchaseListId);//采购清单明细
        BeanUtils.copyProperties(purchaseList, purchaseListVO);//拷贝

        for (PurchaseListDetailsVO purchaseListDetailsVO : purchaseListDetailsVOList) {
            LambdaQueryWrapper<SupplierInformation> supplierInformationLambdaQueryWrapper = new LambdaQueryWrapper<>();
            //赋值供应商名称
            if (purchaseListDetailsVO.getSupplierId() != null) {
                supplierInformationLambdaQueryWrapper.eq(SupplierInformation::getId, purchaseListDetailsVO.getSupplierId());
                SupplierInformation one = supplierInformationService.getOne(supplierInformationLambdaQueryWrapper);
                purchaseListDetailsVO.setSupplierName(one.getSupplierName());
            }
        }
        purchaseListVO.setPurchaseListDetailsVOS(purchaseListDetailsVOList);
        return purchaseListVO;
    }

    @Override
    @Transactional
    //修改采购清单，批量或单个
    public void editById(List<PurchaseListDTO> purchaseListDTOList) {

        for (PurchaseListDTO purchaseListDTO : purchaseListDTOList) {
            PurchaseListDetails purchaseListDetails = purchaseListDetailsService.getById(purchaseListDTO.getId());
            purchaseListDetails.setSupplierId(purchaseListDTO.getSupplierId());//只需修改绑定的供应商ID即可
            purchaseListDetailsService.updateById(purchaseListDetails);
        }
    }

    @Transactional
    @Override//提交采购清单
    public PurchaseList commitById(String id) {

        PurchaseList purchaseList = this.getById(id);
        //将集中采购与分散采购申请的状态改变
        if (purchaseList.getType().equals("采购计划")) {//区分类别，以便查找对应的采购申请
            PurchasingPlan plan = purchasingPlanService.getOne(Wrappers.<PurchasingPlan>query()
                    .eq("purchase_list_id", id));

            PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(plan.getPurchasingPlanId());//采购计划
            //这个采购计划内的所有集中采购申请
            List<CentralizedRequest> centralizedRequestList = centralizedRequestService.list(Wrappers.<CentralizedRequest>query().eq("purchasing_plan_id", plan.getPurchasingPlanId()));
            for (CentralizedRequest centralizedRequest : centralizedRequestList) {
                centralizedRequest.setStatus(4);
                centralizedRequestService.updateById(centralizedRequest);
            }
        } else {
            //除了采购计划，就是分散采购申请
            DecentralizedRequest decentralizedRequest = decentralizedRequestService.getOne(Wrappers.<DecentralizedRequest>query()
                    .eq("purchase_list_id", id));
            decentralizedRequest.setStatus(6);
            decentralizedRequestService.updateById(decentralizedRequest);
        }
        purchaseList.setStatus(1);//提交状态

        LambdaQueryWrapper<PurchaseListDetails> purchaseListDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        purchaseListDetailsLambdaQueryWrapper.eq(PurchaseListDetails::getPurchaseListId, purchaseList.getId());
        List<PurchaseListDetails> purchaseListDetailsList = purchaseListDetailsService.list(purchaseListDetailsLambdaQueryWrapper);//采购清单明细
        //创建入库内容集合
        List<WarehousingContent> warehousingContentList = new ArrayList<>();
        //创建入库记录表
        WarehousingRecordForm warehousingRecordForm = new WarehousingRecordForm();
        //录入入库记录表信息
        warehousingRecordForm.setId(IdWorker.get32UUID().toUpperCase());
        warehousingRecordForm.setPurchaseListId(purchaseList.getId());//关联采购清单
        warehousingRecordForm.setPurchaseNumber(LocalDate.now() + "入库任务");//任务名称，用时间命名
        warehousingRecordForm.setStatus(0);//初始状态

        for (PurchaseListDetails purchaseListDetails : purchaseListDetailsList) {
            //得到该试剂耗材的库存情况
            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(purchaseListDetails.getReagentConsumableId());
            ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getOne(Wrappers.<ReagentConsumableInventory>query()
                    .eq("reagent_consumable_id", reagentConsumables.getReagentConsumableId()));


            WarehousingContent warehousingContent = new WarehousingContent();
            //完善入库明细内容信息
            warehousingContent.setId(IdWorker.get32UUID().toUpperCase());
            warehousingContent.setReagentConsumableName(reagentConsumables.getReagentConsumableName());//试剂耗材名称
            warehousingContent.setTotalQuantity(purchaseListDetails.getPurchaseQuantity());//采购数量
            warehousingContent.setReagentConsumableId(purchaseListDetails.getReagentConsumableId());//试剂耗材ID
            warehousingContent.setWarehousingRecordFormId(warehousingRecordForm.getId());//关联入库表
            warehousingContent.setSupplierId(purchaseListDetails.getSupplierId());//供应商ID
            warehousingContent.setWarehousingQuantity(0);//初始状态
            if (reagentConsumables.getCode() != null) {
                warehousingContent.setCode(reagentConsumables.getCode());//物品编码
            }
            warehousingContent.setPurchaseListDetailsId(purchaseListDetails.getId());//关联采购清单明细

            if (reagentConsumableInventory != null) {//如果仓库存在该试剂耗材
                //所有批次信息
                List<BatchDetails> batchDetailsList = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_inventory_id", reagentConsumableInventory.getReagentConsumableInventoryId())
                        .orderByDesc("create_time"));
                warehousingContent.setWarningValue(reagentConsumableInventory.getWarningValue());
                //如果存在批次信息，则提供上次位置信息，以便用户存放物品
                if (batchDetailsList.size() != 0 && batchDetailsList != null) {
                    warehousingContent.setLastStorageLocation(batchDetailsList.get(0).getLocation());//位置信息
                    warehousingContent.setBoxId(batchDetailsList.get(0).getBoxId());//柜子ID
                    warehousingContent.setLatticeId(batchDetailsList.get(0).getLatticeId());//格子ID
                }
            }
            warehousingContentList.add(warehousingContent);
        }
        if (this.updateById(purchaseList) & warehousingRecordFormService.save(warehousingRecordForm)
                & warehousingContentService.saveBatch(warehousingContentList)) {
            return purchaseList;
        } else {
            return null;
        }
    }

    @Transactional
    @Override//录入采购清单明细
    public PurchaseListDetailsVO addDetails(PurchaseListDTO purchaseListDTO) {

        PurchaseListDetails purchaseListDetails = purchaseListDetailsService.getById(purchaseListDTO.getId());
        purchaseListDetails.setSupplierId(purchaseListDTO.getSupplierId());//供应商ID
        purchaseListDetailsService.updateById(purchaseListDetails);
        PurchaseListDetailsVO purchaseListDetailsVO = purchaseListDetailsService.getPurchaseListDetailsVO(purchaseListDetails.getId());//返回更新后的数据
        return purchaseListDetailsVO;
    }

    @Override//分页查询采购清单列表
    public IPage<PurchaseListPageVO> getPurchaseListVOPage(IPage<PurchaseList> page, @Param(Constants.WRAPPER) QueryWrapper<PurchaseList> qw) {
        IPage<PurchaseListPageVO> purchaseListVOPage = baseMapper.getPurchaseListVOPage(page, qw);
        return purchaseListVOPage;
    }

    @Override
    public List<PurchaseListDetails> addList(String id, Integer type) {
        if (type == 0) {
            List<DecentralizeDetails> decentralizeDetailsList = decentralizeDetailsService.list(new LambdaQueryWrapper<DecentralizeDetails>().eq(DecentralizeDetails::getDecentralizedRequestId, id));
            DecentralizedRequest decentralizedRequest = decentralizedRequestService.getById(id);
            PurchaseList purchaseList = new PurchaseList();
            purchaseList.setName(DateTime.now() + "采购清单");
            purchaseList.setStatus(0);
            purchaseList.setCreateBy(decentralizedRequest.getCreateBy());
            purchaseList.setType("分散采购申请");
            purchaseList.setId(IdWorker.get32UUID().toUpperCase());
            decentralizedRequest.setPurchaseListId(purchaseList.getId());

            ArrayList<PurchaseListDetails> purchaseListDetailsArrayList = new ArrayList<>();
            for (DecentralizeDetails decentralizeDetails : decentralizeDetailsList) {
                if (StrUtil.isNotBlank(decentralizeDetails.getReagentConsumableId())) {
                    ReagentConsumables reagentConsumables = reagentConsumablesService.addReagentConsumables(decentralizeDetails);
                    decentralizeDetails.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                }
                PurchaseListDetails purchaseListDetails = new PurchaseListDetails();
                purchaseListDetails.setId(IdWorker.get32UUID().toUpperCase());
                purchaseListDetails.setReagentConsumableId(decentralizeDetails.getReagentConsumableId());
                purchaseListDetails.setPurchaseListId(purchaseList.getId());
                purchaseListDetails.setPurchaseQuantity(decentralizeDetails.getQuantity());
                decentralizeDetails.setPurchaseListDetailsId(purchaseListDetails.getId());
                purchaseListDetailsArrayList.add(purchaseListDetails);
            }
            if (this.save(purchaseList) &&
                    decentralizedRequestService.updateById(decentralizedRequest) &&
                    decentralizeDetailsService.updateBatchById(decentralizeDetailsList) &&
                    purchaseListDetailsService.saveBatch(purchaseListDetailsArrayList)) {
                return purchaseListDetailsArrayList;
            } else return null;
        } else {

            PurchasingPlan purchasingPlan = purchasingPlanService.getById(id);
            List<ProcurementContent> procurementContentList = procurementContentService.list(new LambdaQueryWrapper<ProcurementContent>().eq(ProcurementContent::getPurchasingPlanId, id));
            PurchaseList purchaseList = new PurchaseList();//创建采购清单
            purchaseList.setId(IdWorker.get32UUID().toUpperCase());
            purchaseList.setName(DateTime.now() + "采购清单");//清单名称
            purchaseList.setStatus(0);//状态
            purchaseList.setType("采购计划");//清单类型，区分分散采购申请和采购计划
            purchaseList.setCreateBy(purchasingPlan.getCreateBy());//清单创建人

            purchasingPlan.setPurchaseListId(purchaseList.getId());//关联采购清单
            ArrayList<PurchaseListDetails> purchaseListDetailsArrayList = new ArrayList<>();

            for (ProcurementContent procurementContent : procurementContentList) {
                PurchaseListDetails purchaseListDetails = new PurchaseListDetails();//采购清单明细
                purchaseListDetails.setPurchaseListId(purchaseList.getId());//关联采购清单
                purchaseListDetails.setPurchaseQuantity(procurementContent.getQuantityPurchased());//采购数量
                purchaseListDetails.setReagentConsumableId(procurementContent.getReagentConsumableId());//采购试剂耗材ID
                purchaseListDetails.setId(IdWorker.get32UUID().toUpperCase());
                purchaseListDetails.setProcurementContentId(procurementContent.getProcurementContentId());//关联采购计划明细
                purchaseListDetailsArrayList.add(purchaseListDetails);
            }
            if (this.save(purchaseList) &&
                    purchasingPlanService.updateById(purchasingPlan) &&
                    procurementContentService.updateBatchById(procurementContentList) &&
                    purchaseListDetailsService.saveBatch(purchaseListDetailsArrayList)) {
                return purchaseListDetailsArrayList;
            } else return null;
        }
    }
}
