package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.feign.RemoteTemplate2htmlService;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.ReagentConsumableInventoryDTO;
import digital.laboratory.platform.reagent.utils.PageUtils;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.ReagentConsumableInventoryMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import digital.laboratory.platform.sys.entity.CabinetCell;
import digital.laboratory.platform.sys.entity.CellAndStoreSupplyData;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.feign.RemoteCabinetService;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import feign.Response;
import io.seata.spring.annotation.GlobalTransactional;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.swing.*;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 试剂耗材库存服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 服务实现类
 */
@Service
@SuppressWarnings("all")
public class ReagentConsumableInventoryServiceImpl extends ServiceImpl<ReagentConsumableInventoryMapper, ReagentConsumableInventory> implements ReagentConsumableInventoryService {
    @Autowired
    private BatchDetailsService batchDetailsService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private RemoteTemplate2htmlService remoteTemplate2htmlService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private FixedValueReportService fixedValueReportService;

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private InstructionBookService instructionBookService;

    @Autowired
    private RemoteCabinetService remoteCabinetService;

    @Autowired
    private ReagentConsumableStashService reagentConsumableStashService;

    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;

    @Autowired
    private LocationInfoService locationInfoService;

    @Autowired
    private RemoteUserService remoteUserService;


    @Autowired
    private OssFile ossFile;

    @Override//标准物质标准物质管理列表
    public IPage<ReagentConsumableInventoryVO> getReagentConsumableInventoryRMVOList(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw, Integer warning) {


        if (warning != null) {

            if (warning == 2) {

                List<ReagentConsumableInventoryVO> reagentConsumableInventoryVOList = new ArrayList<>();

                List<BatchDetails> list = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("warning_information", "即将过期").or().eq("warning_information", "已过期").or().eq("warning_information", "即将到达存储期限").or().eq("warning_information", "已超过存储期限"));

                if (list.size() != 0) {
                    for (BatchDetails batchDetails : list) {

                        ReagentConsumableInventory byId = this.getById(batchDetails.getReagentConsumableInventoryId());
                        ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();
                        BeanUtils.copyProperties(byId, reagentConsumableInventoryVO);
                        reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId()));
                        if (!reagentConsumableInventoryVOList.contains(reagentConsumableInventoryVO)) {
                            if (reagentConsumableInventoryVO.getCategory().equals("标准物质")) {
                                reagentConsumableInventoryVOList.add(reagentConsumableInventoryVO);
                            }
                        }

                    }
                }
                PageUtils pageUtils = new PageUtils();

                return pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOList);
            }
        }


        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryVOPage = baseMapper.getReagentConsumableInventoryVOPage(page, qw);

        List<ReagentConsumableInventoryVO> records = reagentConsumableInventoryVOPage.getRecords();

        for (ReagentConsumableInventoryVO reagentConsumableInventoryVO : records) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId());
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);
        }
        return reagentConsumableInventoryVOPage;
    }

    @Override//试剂耗材管理列表
    public IPage<ReagentConsumableInventoryVO> getReagentConsumableInventoryREVOList(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw, Integer warning) {
        if (warning != null && warning == 2) {
            List<ReagentConsumableInventoryVO> reagentConsumableInventoryVOList = new ArrayList<>();
            List<BatchDetails> list = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("warning_information", "即将过期").or().eq("warning_information", "已过期").or().eq("warning_information", "即将到达存储期限").or().eq("warning_information", "已超过存储期限"));
            //查找具有到期批次物品的试剂耗材类信息
            if (list != null && list.size() != 0) {
                for (BatchDetails batchDetails : list) {
                    ReagentConsumableInventory reagentConsumableInventory = this.getOne(new LambdaQueryWrapper<ReagentConsumableInventory>().eq(ReagentConsumableInventory::getReagentConsumableInventoryId, batchDetails.getReagentConsumableInventoryId()).ne(ReagentConsumableInventory::getCategory, "标准物质").ne(ReagentConsumableInventory::getCategory, "标准储备溶液"));
                    if (reagentConsumableInventory != null) {
                        ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();
                        BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryVO);
                        reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId()));
                        reagentConsumableInventoryVOList.add(reagentConsumableInventoryVO);
                    }
                }
            }
            PageUtils pageUtils = new PageUtils();
            return pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOList);
        }
        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryVOPage = baseMapper.getReagentConsumableInventoryVOPage(page, qw);
        List<ReagentConsumableInventoryVO> records = reagentConsumableInventoryVOPage.getRecords();
        for (ReagentConsumableInventoryVO reagentConsumableInventoryVO : records) {
            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId());
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);
        }
        return reagentConsumableInventoryVOPage;
    }


    @Override//入库增加库存
    public ReagentConsumableInventory addById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = this.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() + quantity);


        if (this.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存添加失败"));
    }

    @Override//领用减少库存
    public ReagentConsumableInventory reduceById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = this.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() - quantity);

        if (one.getTotalQuantity() <= one.getWarningValue()) {

            one.setWarningInformation("库存不足");
        }

        if (this.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存减少失败"));
    }


    @Override//分页查询标准物质
    public IPage<ReferenceMaterialFullVO> getAllList(Page page, QueryWrapper<ReferenceMaterial> qw, Integer status, String number) {
        //如果扫码

        if (StrUtil.isNotBlank(number)) {

            ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(Wrappers.<ReferenceMaterial>query().eq("number", number));

            if (referenceMaterial != null) {

                ReagentConsumableInventoryFullVO byCode = this.getByCode(referenceMaterial.getId());

                List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOS = new ArrayList<>();

                reagentConsumableInventoryFullVOS.add(byCode);

                PageUtils pageUtils = new PageUtils();

                Page pages = pageUtils.getPages(page.getCurrent(), page.getSize(), reagentConsumableInventoryFullVOS);

                return pages;

            }
        }

        IPage<ReferenceMaterialFullVO> referenceMaterialPage = baseMapper.getReferenceMaterialPage(page, qw);

        return referenceMaterialPage;
    }


    @Override//通过ID，获取标准物质所有信息
    public ReagentConsumableInventoryFullVO getByCode(String id) {

        ReferenceMaterial referenceMaterial = referenceMaterialService.getById(id);

        if (referenceMaterial == null) {
            throw new RuntimeException(String.format("未能查询到该ID对应的标准物质"));
        }
        ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

        ReagentConsumableInventory byId = this.getById(referenceMaterial.getReagentConsumableInventoryId());

        BeanUtils.copyProperties(byId, reagentConsumableInventoryFullVO);
        reagentConsumableInventoryFullVO.setNumber(referenceMaterial.getNumber());
        reagentConsumableInventoryFullVO.setBatchDetailsId(referenceMaterial.getBatchDetailsId());
        BatchDetails batchDetails = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());
        SupplierInformation supplierInformation = supplierInformationService.getById(batchDetails.getSupplierId());
        LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionLambdaQueryWrapper = new LambdaQueryWrapper<>();
        standardReserveSolutionLambdaQueryWrapper.eq(StandardReserveSolution::getReferenceId, referenceMaterial.getId());
        StandardReserveSolution one1 = standardReserveSolutionService.getOne(standardReserveSolutionLambdaQueryWrapper);

        reagentConsumableInventoryFullVO.setSupplierName(supplierInformation.getSupplierName());
        reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());
        reagentConsumableInventoryFullVO.setBatchNumber(batchDetails.getBatchNumber());
        reagentConsumableInventoryFullVO.setManufacturerId(batchDetails.getSupplierId());
        reagentConsumableInventoryFullVO.setPurchaseTime(batchDetails.getPurchaseTime());
        reagentConsumableInventoryFullVO.setReferenceMaterialId(referenceMaterial.getId());
        reagentConsumableInventoryFullVO.setReferenceMaterialStatus(referenceMaterial.getStatus());
        reagentConsumableInventoryFullVO.setFixedResult(referenceMaterial.getFixedResult());

        List<StandardMaterialApplication> list = standardMaterialApplicationService.list(new LambdaQueryWrapper<StandardMaterialApplication>().eq(StandardMaterialApplication::getReferenceMaterialId, id).orderByDesc(StandardMaterialApplication::getDateOfClaim));

        R<SysUser> userR = remoteUserService.innerGetById(list.get(0).getRecipientId());
        reagentConsumableInventoryFullVO.setHolderName(userR.getData().getName());

        if (one1 != null) {
            reagentConsumableInventoryFullVO.setConfigurationConcentration(one1.getConfigurationConcentration());
            reagentConsumableInventoryFullVO.setConfigurationDate(one1.getConfigurationDate());
            reagentConsumableInventoryFullVO.setSolutionNumbering(referenceMaterial.getNumber());
            reagentConsumableInventoryFullVO.setValidityPeriod(one1.getValidityPeriod());
        }
        return reagentConsumableInventoryFullVO;
    }

    //分页查询试剂耗材
    @Override
    public IPage<ReagentConsumableInventoryFullVO> getAllRM(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw) {

        List<ReagentConsumableInventory> list = this.list(qw);

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetails> list1 = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_inventory_id", reagentConsumableInventory.getReagentConsumableInventoryId()).eq("service_status", 1));

            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                reagentConsumableInventoryFullVO.setSupplierName(supplierInformationService.getById(batchDetails.getSupplierId()).getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOS.add(reagentConsumableInventoryFullVO);
            }
        }
        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryFullVOS);

        return pages;

    }

    @Override
    public List<ReagentConsumableInventoryFullVO> getReagentConsumableInventoryFull(QueryWrapper<ReagentConsumableInventory> qw) {
        List<ReagentConsumableInventory> list = this.list(qw);
//        return list;

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();
        //找出对应的仓库信息
        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            batchDetailsLambdaQueryWrapper.eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId()).or().eq(BatchDetails::getServiceStatus, 1);

            List<BatchDetails> list1 = batchDetailsService.list(batchDetailsLambdaQueryWrapper);
            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }
        return reagentConsumableInventoryFullVOList;
    }


    @Override
    public void referenceMaterialTablePDF(List<ReferenceMaterialVO> voList, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("referenceMaterialTablePDF.................");

        int x = 1;

        for (ReferenceMaterialVO referenceMaterialVO : voList) {

            referenceMaterialVO.setI(x);
            x++;
        }
        //-----------------------------
        // 生成 word 版本的 采购目录
        String applyFileName = "标准物质一览表-" + id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "标准物质一览表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);
        builder.bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(new HashMap<String, Object>() {{
            put("voList", voList);
            put("year", year);
        }});
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "referenceMaterial" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }

    //试剂/耗材扫描物品编码进行持久化
    @Override
    public void setCode(String id, String code) {

        ReagentConsumables byId = reagentConsumablesService.getById(id);

        ReagentConsumableInventory one = this.getOne(Wrappers.<ReagentConsumableInventory>query().eq("reagent_consumable_id", id));

        if (byId != null & one != null) {
            byId.setCode(code);
            one.setCode(code);

            this.updateById(one);
            reagentConsumablesService.updateById(byId);
        }

    }

    @Override
    public void setRMCode(String id, String code) {

        ReferenceMaterial referenceMaterial = referenceMaterialService.getById(id);

        if (referenceMaterial != null) {

            referenceMaterial.setCode(code);

            referenceMaterialService.updateById(referenceMaterial);
        }
    }

    @Override
//标准物质二维码打印
    public String buildCodeLabelContent(String id) {
        //录入二维码信息
        ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(id);

        referenceMaterialServiceById.setCode(referenceMaterialServiceById.getNumber());
        referenceMaterialService.updateById(referenceMaterialServiceById);

        if (referenceMaterialServiceById == null) {

            throw new RuntimeException(String.format("请先入库后再进行打印条码"));
        }
        //获取打印信息
        Map<String, Object> data = new HashMap<>();
        referenceMaterialService.updateById(referenceMaterialServiceById);
        ReagentConsumables byId = reagentConsumablesService.getById(referenceMaterialServiceById.getReagentConsumableId());
        String reagentConsumableName = byId.getReagentConsumableName();
        ReferenceMaterialVO referenceMaterial = new ReferenceMaterialVO();
        BatchDetails byId1 = batchDetailsService.getById(referenceMaterialServiceById.getBatchDetailsId());
        BeanUtils.copyProperties(referenceMaterialServiceById, referenceMaterial);
        //完善打印信息
        referenceMaterial.setReferenceMaterialName(byId.getReagentConsumableName());
        referenceMaterial.setTime("到期:" + byId1.getExpirationDate());
        data.put("referenceMaterial", referenceMaterial);
        String templateFileName = "标准物质标签模板.vm";

        return remoteTemplate2htmlService.getHtml(templateFileName, data);
    }

    @Override
    public String printSolutionTag(String id) {

        StandardReserveSolutionVO standardReserveSolutionVOById = standardReserveSolutionService.getStandardReserveSolutionVOById(id);
        Map<String, Object> data = new HashMap<>();
        data.put("standardReserveSolution", standardReserveSolutionVOById);
        String templateFileName = "标准储备溶液标签模板.vm";
        return remoteTemplate2htmlService.getHtml(templateFileName, data);
    }

    @Override
    public List<UserVO> getUserList() {

        return baseMapper.getUserList();
    }

    @Override
    @Transactional//盘点导入，需提供柜子系统数据
    public void inventoryEntry(List<ReagentConsumableInventoryDTO> reagentConsumableInventoryDTOList) {

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);

        for (ReagentConsumableInventoryDTO reagentConsumableInventoryDTO : reagentConsumableInventoryDTOList) {
            Integer quantity = reagentConsumableInventoryDTO.getTotalQuantity() * reagentConsumableInventoryDTO.getPackagedCopies();
            SupplierInformation supplier = supplierInformationService.getOne(Wrappers.<SupplierInformation>query().eq("supplier_name", reagentConsumableInventoryDTO.getSupplierName()));
            R<CabinetCell> cabinetCellR = remoteCabinetService.queryCellByRoomNoAndLocation(reagentConsumableInventoryDTO.getRoomNo(), reagentConsumableInventoryDTO.getLocation());
            CabinetCell cabinetCell = cabinetCellR.getData();
            String cabinetId = cabinetCellR.getData().getCabinetId();
            String cellName = baseMapper.getCellName(cabinetId);
            String location = "F" + reagentConsumableInventoryDTO.getRoomNo().charAt(0) + "-" + reagentConsumableInventoryDTO.getRoomNo() + "_" + cellName + "_" + reagentConsumableInventoryDTO.getLocation();
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getStandardValueOrPurity())) {
                reagentConsumableInventoryDTO.setStandardValueOrPurity("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getDeviationOrUncertainty())) {
                reagentConsumableInventoryDTO.setDeviationOrUncertainty("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getEnglishName())) {
                reagentConsumableInventoryDTO.setEnglishName("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getCasNumber())) {
                reagentConsumableInventoryDTO.setCasNumber("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getCode())) {
                reagentConsumableInventoryDTO.setCode("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getPurityGrade())) {
                reagentConsumableInventoryDTO.setPurityGrade("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getRemark())) {
                reagentConsumableInventoryDTO.setRemark("");
            }
            if (!StrUtil.isNotBlank(reagentConsumableInventoryDTO.getAlias())) {
                reagentConsumableInventoryDTO.setAlias("");
            }

            ReagentConsumableInventory one = this.getOne(Wrappers.<ReagentConsumableInventory>query().eq("reagent_consumable_name", reagentConsumableInventoryDTO.getReagentConsumableName()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getBrand()), "brand", reagentConsumableInventoryDTO.getBrand()).eq("category", reagentConsumableInventoryDTO.getCategory()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getSpecificationAndModel()), "specification_and_model", reagentConsumableInventoryDTO.getSpecificationAndModel()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getStandardValueOrPurity()), "standard_value_or_purity", reagentConsumableInventoryDTO.getStandardValueOrPurity()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getCasNumber()), "cas_number", reagentConsumableInventoryDTO.getCasNumber()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getSpecies()), "species", reagentConsumableInventoryDTO.getSpecies()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getEnglishName()), "english_name", reagentConsumableInventoryDTO.getEnglishName()).eq(reagentConsumableInventoryDTO.getPackagedCopies() != null, "packaged_copies", reagentConsumableInventoryDTO.getPackagedCopies()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getAlias()), "alias", reagentConsumableInventoryDTO.getAlias()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getPurityGrade()), "purity_grade", reagentConsumableInventoryDTO.getPurityGrade()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getDeviationOrUncertainty()), "deviation_or_uncertainty", reagentConsumableInventoryDTO.getDeviationOrUncertainty()).eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getMinimumUnit()), "minimum_unit", reagentConsumableInventoryDTO.getMinimumUnit()));
            BatchDetails batchDetails = new BatchDetails();
            batchDetails.setServiceStatus(1);
            batchDetails.setQuantity(quantity);
            batchDetails.setBatch(year + "-1");
            if (supplier == null) {
                SupplierInformation supplierInformation = new SupplierInformation();
                supplierInformation.setSupplierName(reagentConsumableInventoryDTO.getSupplierName());
                supplierInformation.setAcceptableCondition("合格");
                supplierInformation.setId(IdWorker.get32UUID());
                supplierInformationService.save(supplierInformation);
                batchDetails.setSupplierId(supplierInformation.getId());

            } else {
                batchDetails.setSupplierId(supplier.getId());
            }
            //如果没有生产日期/到期时间，就自动生成
            if (reagentConsumableInventoryDTO.getDateOfProduction() != null) {
                batchDetails.setDateOfProduction(reagentConsumableInventoryDTO.getDateOfProduction());
            } else {
                batchDetails.setDateOfProduction(LocalDate.now());
            }
            if (reagentConsumableInventoryDTO.getExpirationDate() != null) {
                batchDetails.setExpirationDate(reagentConsumableInventoryDTO.getExpirationDate());
            } else {
                batchDetails.setExpirationDate(LocalDate.now().plusYears(99));
            }
            batchDetails.setLatticeId(cabinetCell.getId());
            batchDetails.setBoxId(cabinetCell.getCabinetId());
            batchDetails.setLocation(location);
            batchDetails.setPurchaseTime(LocalDateTime.now());
            batchDetails.setBatchNumber("/");
            batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
            batchDetails.setRoomNo(reagentConsumableInventoryDTO.getRoomNo());
            if (one != null) {
                List<BatchDetails> batchDetailsList = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_inventory_id", one.getReagentConsumableInventoryId()));
                if (batchDetailsList != null & batchDetailsList.size() > 0) {
                    int x = batchDetailsList.size() + 1;
                    batchDetails.setBatch("2023-" + x);
                    one.setTotalQuantity(one.getTotalQuantity() + quantity);
                    this.updateById(one);
                }
                batchDetails.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                ReagentConsumables reagentConsumables = reagentConsumablesService.getById(one.getReagentConsumableId());
                if (!one.getCategory().equals("标准物质")) {
                    ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getOne(Wrappers.<ReagentConsumableStash>query().eq("room_no", reagentConsumableInventoryDTO.getRoomNo()).eq("reagent_consumable_id", reagentConsumables.getReagentConsumableId()));
                    if (reagentConsumableStash != null) {
                        reagentConsumableStash.setTotalQuantity(reagentConsumableStash.getTotalQuantity() + quantity);
                        batchDetails.setReagentConsumableStashId(reagentConsumableStash.getId());
                        reagentConsumableStashService.updateById(reagentConsumableStash);
                    } else {
                        ReagentConsumableStash consumableStash = new ReagentConsumableStash();
                        BeanUtils.copyProperties(reagentConsumableInventoryDTO, consumableStash);
                        consumableStash.setId(IdWorker.get32UUID().toUpperCase());
                        consumableStash.setTotalQuantity(quantity);
                        consumableStash.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                        batchDetails.setReagentConsumableStashId(consumableStash.getId());
                        reagentConsumableStashService.save(consumableStash);
                    }
                }
                if (reagentConsumableInventoryDTO.getCategory().equals("标准物质")) {
                    ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                    referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                    referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                    referenceMaterial.setLocation(location);
                    referenceMaterial.setStatus(0);
                    referenceMaterial.setLatticeId(cabinetCell.getId());
                    referenceMaterial.setBoxId(cabinetCell.getCabinetId());
                    String roomNo = reagentConsumableInventoryDTO.getRoomNo();
                    referenceMaterial.setNumber(reagentConsumableInventoryDTO.getAlias() + "-" + year + "-" + roomNo.charAt(0) + "-1");//自动生成编号
                    referenceMaterial.setCode(referenceMaterial.getNumber());
                    referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                    referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                    referenceMaterial.setReagentConsumableName(reagentConsumableInventoryDTO.getReagentConsumableName());
                    referenceMaterial.setMass(reagentConsumableInventoryDTO.getMass());
                    referenceMaterial.setReferenceMaterialUnit(reagentConsumableInventoryDTO.getReferenceMaterialUnit());
                    referenceMaterialService.save(referenceMaterial);
                    this.updateCabinet(cabinetCell.getId(), referenceMaterial.getId(), 1);
                }
                this.updateCabinet(cabinetCell.getId(), reagentConsumables.getReagentConsumableId(), 1);

            } else {
                ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();
                BeanUtils.copyProperties(reagentConsumableInventoryDTO, reagentConsumableInventory);
                reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());
                reagentConsumableInventory.setWarningValue(1);
                if (reagentConsumableInventoryDTO.getPackages() != null) {
                    if (reagentConsumableInventoryDTO.getSpecificationAndModel().equals("/")) {
                        reagentConsumableInventory.setSpecificationAndModel(reagentConsumableInventoryDTO.getPackages());
                    } else {
                        reagentConsumableInventory.setSpecificationAndModel(reagentConsumableInventoryDTO.getSpecificationAndModel() + "," + reagentConsumableInventoryDTO.getPackages());
                    }
                }
                batchDetails.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
                ReagentConsumables reagentConsumables = reagentConsumablesService.addReagentConsumables(reagentConsumableInventory);
                reagentConsumableInventory.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                reagentConsumableInventory.setTotalQuantity(quantity);
                this.save(reagentConsumableInventory);
                this.updateCabinet(cabinetCell.getId(), reagentConsumables.getReagentConsumableId(), 1);

                if (!reagentConsumableInventory.getCategory().equals("标准物质")) {
                    ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getOne(Wrappers.<ReagentConsumableStash>query().eq("room_no", reagentConsumableInventoryDTO.getRoomNo()).eq("reagent_consumable_id", reagentConsumables.getReagentConsumableId()));
                    if (reagentConsumableStash != null) {
                        reagentConsumableStash.setTotalQuantity(reagentConsumableStash.getTotalQuantity() + quantity);
                        batchDetails.setReagentConsumableStashId(reagentConsumableStash.getId());
                        reagentConsumableStashService.updateById(reagentConsumableStash);
                    } else {
                        ReagentConsumableStash consumableStash = new ReagentConsumableStash();
                        BeanUtils.copyProperties(reagentConsumableInventoryDTO, consumableStash);
                        consumableStash.setId(IdWorker.get32UUID().toUpperCase());
                        consumableStash.setTotalQuantity(quantity);
                        consumableStash.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                        batchDetails.setReagentConsumableStashId(consumableStash.getId());
                        reagentConsumableStashService.save(consumableStash);
                    }
                }
                if (reagentConsumableInventoryDTO.getCategory().equals("标准物质")) {
                    ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                    referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                    referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
                    referenceMaterial.setLocation(location);
                    referenceMaterial.setStatus(0);
                    referenceMaterial.setLatticeId(cabinetCell.getId());
                    referenceMaterial.setBoxId(cabinetCell.getCabinetId());
                    referenceMaterial.setReagentConsumableName(reagentConsumableInventoryDTO.getReagentConsumableName());
                    String roomNo = reagentConsumableInventoryDTO.getRoomNo();
                    referenceMaterial.setNumber(reagentConsumableInventoryDTO.getAlias() + "-" + year + "-" + roomNo.charAt(0) + "-1");//自动生成编号
                    referenceMaterial.setCode(referenceMaterial.getNumber());
                    referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                    referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                    referenceMaterial.setMass(reagentConsumableInventoryDTO.getMass());
                    referenceMaterial.setReferenceMaterialUnit(reagentConsumableInventoryDTO.getReferenceMaterialUnit());
                    referenceMaterialService.save(referenceMaterial);
                    this.updateCabinet(cabinetCell.getId(), referenceMaterial.getId(), 1);
                }
                this.updateCabinet(cabinetCell.getId(), reagentConsumables.getReagentConsumableId(), 1);
            }
            batchDetailsService.save(batchDetails);
        }
    }

    @Override
    //更新柜子系统方法，柜子ID、物品ID、入库OR出库 (opCode 1 or -1)
    public void updateCabinet(String latticeId, String itemId, int opCode) {
        ArrayList<CellAndStoreSupplyData> cellAndStoreSupplyDataArrayList = new ArrayList<>();//柜子数组
        CellAndStoreSupplyData cellAndStoreSupplyData = new CellAndStoreSupplyData();
        cellAndStoreSupplyData.setCellStatus("1");
        cellAndStoreSupplyData.setCellId(latticeId);
        ArrayList<String> arrayList = new ArrayList<>();
        arrayList.add(itemId);
        cellAndStoreSupplyData.setStoreSupplyList(arrayList);
        cellAndStoreSupplyDataArrayList.add(cellAndStoreSupplyData);
        remoteCabinetService.takeOrSaveGoodsFromCabinet(cellAndStoreSupplyDataArrayList, opCode);
    }


    @Override
    public List<OrgFullVO> getOrgList() {

        List<OrgVO> orgList = baseMapper.getOrgList();
        ArrayList<OrgFullVO> orgFullVOS = new ArrayList<>();
        for (OrgVO orgVO : orgList) {
            if (orgVO.getOrgType() != null && orgVO.getOrgType().equals("province") && !(orgVO.getAddress().substring(0, 2).equals("陕西"))) {
                OrgFullVO orgFullVO = new OrgFullVO();
                orgFullVO.setUsername("P" + orgVO.getAreaCode());
                String[] roles = {"10010", "10015", "10020", "10130", "23020", "23030"};
                orgFullVO.setRoleIds(roles);
                orgFullVO.setPassword("123456");
                orgFullVO.setOrgId(orgVO.getOrgId());
                orgFullVO.setGender(1);
                orgFullVOS.add(orgFullVO);
            }
        }
        return orgFullVOS;
    }

    @Override
    @GlobalTransactional
    public boolean updateLocation(BatchDetails batchDetails) {

        BatchDetails oldBatchDetails = batchDetailsService.getById(batchDetails.getBatchDetailsId());
        String oldLocation = oldBatchDetails.getLocation();
        this.updateCabinet(oldBatchDetails.getLatticeId(), oldBatchDetails.getReagentConsumableInventoryId(), -1);//清除原来位置信息ID
        //需同步至试剂耗材仓库库存表
        ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getById(oldBatchDetails.getReagentConsumableStashId());
        oldBatchDetails.setLocation(batchDetails.getLocation());
        oldBatchDetails.setBoxId(batchDetails.getBoxId());
        oldBatchDetails.setLatticeId(batchDetails.getLatticeId());
        String[] split = batchDetails.getLocation().split("_");
        String roomNo = split[1];
        oldBatchDetails.setRoomNo(roomNo);
        reagentConsumableStash.setRoomNo(roomNo);

        if (batchDetailsService.updateById(oldBatchDetails)) {
            this.updateCabinet(oldBatchDetails.getLatticeId(), oldBatchDetails.getReagentConsumableInventoryId(), 1);
            reagentConsumableStashService.updateById(reagentConsumableStash);
            locationInfoService.addInfo(oldBatchDetails.getBatchDetailsId(), oldLocation, batchDetails.getLocation());
            return true;
        } else return false;

    }

    @Override
    public Map<String, Object> getBatchVO(String id) {

        BatchDetails batchDetails = batchDetailsService.getById(id);
        if (batchDetails == null) {
            throw new RuntimeException(String.format("没有找到对应的物品信息"));
        }
        BatchDetailsVO batchDetailsVO = new BatchDetailsVO();
        BeanUtils.copyProperties(batchDetails, batchDetailsVO);
        batchDetailsVO.setSupplierName(supplierInformationService.getById(batchDetails.getSupplierId()).getSupplierName());
        ReagentConsumableInventory reagentConsumableInventory = this.getById(batchDetails.getReagentConsumableInventoryId());
        List<LocationInfoVO> locationInfoVOList = locationInfoService.getListById(id);

        HashMap<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("batchDetails", batchDetailsVO);
        objectObjectHashMap.put("reagentConsumableInventory", reagentConsumableInventory);
        objectObjectHashMap.put("locationInfoVOList", locationInfoVOList);

        return objectObjectHashMap;
    }


    @Override
    public String getExcel() throws IOException {

        String Path = "C:\\tmp\\upload\\";

        List<ReagentConsumableInventoryFullVO> reagentConsumableList = baseMapper.getReagentConsumableList();

        HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFCellStyle cellStyle = workbook.createCellStyle();
        HSSFCellStyle cellStyle1 = workbook.createCellStyle();
        cellStyle1.setAlignment(HorizontalAlignment.CENTER);
        HSSFFont font1 = workbook.createFont();
        font1.setFontHeightInPoints((short) 10);
        cellStyle1.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        cellStyle1.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        cellStyle1.setFont(font1);
        HSSFCellStyle cellStyle2 = workbook.createCellStyle();
        cellStyle2.setAlignment(HorizontalAlignment.CENTER);
        HSSFFont font2 = workbook.createFont();
        font2.setFontName("微软雅黑");
        cellStyle2.setFont(font2);
//        cellStyle2.setWrapText(true);

        HSSFSheet workbookSheet = workbook.createSheet("试剂耗材库存表");
        workbookSheet.setColumnWidth(0, 15000);
        workbookSheet.setColumnWidth(1, 3500);
        workbookSheet.setColumnWidth(2, 4000);
        workbookSheet.setColumnWidth(3, 15000);
        workbookSheet.setColumnWidth(4, 3500);
        workbookSheet.setColumnWidth(5, 3500);
        workbookSheet.setColumnWidth(6, 3500);
        workbookSheet.setColumnWidth(7, 3500);
        workbookSheet.setColumnWidth(8, 3500);
        workbookSheet.setColumnWidth(9, 3500);
        workbookSheet.setColumnWidth(10, 3500);
        workbookSheet.setColumnWidth(11, 3500);
        workbookSheet.setColumnWidth(12, 10000);
        HSSFFont font = workbook.createFont();
        HSSFRow row0 = workbookSheet.createRow(0);
        cellStyle.setAlignment(HorizontalAlignment.CENTER);

        row0.setRowStyle(cellStyle);
        for (int i = 0; i < 13; i++) {
            HSSFCell cell = row0.createCell(i);
            if (i == 0) {
                LocalDateTime dateTime = LocalDateTime.now();
                cell.setCellValue(LocalDateTimeUtil.format(dateTime, "yyyy 年 MM 月 dd 日") + "试剂耗材实时库存报表");
                font.setFontHeightInPoints((short) 28);
                font.setColor((short) 25);
//                font.setBoldweight((short) 1);
                //设置字体斜体
                cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
                cellStyle.setFillForegroundColor(IndexedColors.TAN.getIndex());
                font.setItalic(true);
                cellStyle.setFont(font);
                cell.setCellStyle(cellStyle);
            }
        }
        CellRangeAddress cellRangeAddress = new CellRangeAddress(0, 0, 0, 12);
        workbookSheet.addMergedRegion(cellRangeAddress);
        HSSFRow row = workbookSheet.createRow(1);
        HSSFCell cell0 = row.createCell(0);
        cell0.setCellValue("试剂耗材名称");
        HSSFCell cell1 = row.createCell(1);
        cell1.setCellValue("类型");
        HSSFCell cell2 = row.createCell(2);
        cell2.setCellValue("品牌");
        cell2.setCellStyle(cellStyle1);
        HSSFCell cell3 = row.createCell(3);
        cell3.setCellValue("规格型号");
        HSSFCell cell4 = row.createCell(4);
        cell4.setCellValue("批次");
        HSSFCell cell5 = row.createCell(5);
        cell5.setCellValue("房间号");
        HSSFCell cell6 = row.createCell(6);
        cell6.setCellValue("位置信息");
        HSSFCell cell7 = row.createCell(7);
        cell7.setCellValue("入库时间");
        HSSFCell cell8 = row.createCell(8);
        cell8.setCellValue("数量");
        HSSFCell cell9 = row.createCell(9);
        cell9.setCellValue("实际房间");
        HSSFCell cell10 = row.createCell(10);
        cell10.setCellValue("实际位置");
        HSSFCell cell111 = row.createCell(11);
        cell111.setCellValue("实际数量");
        HSSFCell cell12 = row.createCell(12);
        cell12.setCellValue("批次ID(请勿修改)");

        cell0.setCellStyle(cellStyle1);
        cell1.setCellStyle(cellStyle1);
        cell2.setCellStyle(cellStyle1);
        cell3.setCellStyle(cellStyle1);
        cell4.setCellStyle(cellStyle1);
        cell5.setCellStyle(cellStyle1);
        cell6.setCellStyle(cellStyle1);
        cell7.setCellStyle(cellStyle1);
        cell8.setCellStyle(cellStyle1);
        cell9.setCellStyle(cellStyle1);
        cell10.setCellStyle(cellStyle1);
        cell111.setCellStyle(cellStyle1);
        cell12.setCellStyle(cellStyle1);

        int index = 2;
        for (ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO : reagentConsumableList) {
            HSSFRow row1 = workbookSheet.createRow(index);
            HSSFCell cell00 = row1.createCell(0);
            cell00.setCellStyle(cellStyle2);
            cell00.setCellValue(reagentConsumableInventoryFullVO.getReagentConsumableName());
            HSSFCell cell11 = row1.createCell(1);
            cell11.setCellValue(reagentConsumableInventoryFullVO.getCategory());
            HSSFCell cell22 = row1.createCell(2);
            cell22.setCellValue(reagentConsumableInventoryFullVO.getBrand());
            HSSFCell cell33 = row1.createCell(3);
            cell33.setCellValue(reagentConsumableInventoryFullVO.getSpecificationAndModel());
            HSSFCell cell44 = row1.createCell(4);
            cell44.setCellValue(reagentConsumableInventoryFullVO.getBatch());
            HSSFCell cell55 = row1.createCell(5);
            cell55.setCellValue(reagentConsumableInventoryFullVO.getRoomNo());
            HSSFCell cell66 = row1.createCell(6);
            String location = reagentConsumableInventoryFullVO.getLocation();
            String substring = location.substring(location.length() - 4, location.length());
            cell66.setCellValue(substring);
            HSSFCell cell77 = row1.createCell(7);
            cell77.setCellValue(LocalDateTimeUtil.format(reagentConsumableInventoryFullVO.getBatchPurchaseTime(), "yyyy-MM-dd"));
            HSSFCell cell88 = row1.createCell(8);
            cell88.setCellValue(reagentConsumableInventoryFullVO.getQuantity());
            HSSFCell cell99 = row1.createCell(12);
            cell99.setCellValue(reagentConsumableInventoryFullVO.getBatchDetailsId());
            index++;
            cell00.setCellStyle(cellStyle2);
            cell11.setCellStyle(cellStyle2);
            cell22.setCellStyle(cellStyle2);
            cell33.setCellStyle(cellStyle2);
            cell44.setCellStyle(cellStyle2);
            cell55.setCellStyle(cellStyle2);
            cell66.setCellStyle(cellStyle2);
            cell77.setCellStyle(cellStyle2);
            cell88.setCellStyle(cellStyle2);
            cell99.setCellStyle(cellStyle2);
        }
        LocalDateTime now = LocalDateTime.now();
        String format = LocalDateTimeUtil.format(now, "yyyy-MM-dd");
        String fileName = format + "试剂耗材库存表.xls";
        FileOutputStream fileOutputStream = new FileOutputStream(Path + fileName);
        workbook.write(fileOutputStream);
        fileOutputStream.close();
        File file = new File(Path + fileName);
        FileItem fileItem = this.getMultipartFile(file, file.getName());
        MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
        String ossPath = "document" + "/" + "reagentConsumableInventory";
        boolean ret = ossFile.fileUpload(multipartFile, ossPath);
        if (ret) {
            Files.delete(Paths.get(Path + fileName));
            return fileName;
        }
        return null;
    }

    private FileItem getMultipartFile(File file, String fieldName) {
        FileItemFactory factory = new DiskFileItemFactory(16, null);
        FileItem item = factory.createItem(fieldName, "text/plain", true, file.getName());
        int bytesRead = 0;
        byte[] buffer = new byte[8192];
        try {
            FileInputStream fis = new FileInputStream(file);
            OutputStream os = item.getOutputStream();
            while ((bytesRead = fis.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            fis.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return item;
    }

    @Transactional
    @Override
    public boolean checkObj(List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList) {
        ArrayList<ReagentConsumableInventory> reagentConsumableInventoryArrayList = new ArrayList<>();
        ArrayList<ReagentConsumableStash> reagentConsumableStashArrayList = new ArrayList<>();
        ArrayList<BatchDetails> batchDetailsArrayList = new ArrayList<>();
        int index = 1;

        long s = System.currentTimeMillis();
        for (ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO : reagentConsumableInventoryFullVOList) {
            long start = System.currentTimeMillis();
            boolean loop = false;
            String practicalRoomNo = reagentConsumableInventoryFullVO.getPracticalRoomNo();
            String practicalLocation = reagentConsumableInventoryFullVO.getPracticalLocation();
            Integer practicalQuantity = reagentConsumableInventoryFullVO.getPracticalQuantity();

            //若没修改，则无需操作数据库，下面的判断是为了减轻查询压力
            if (practicalQuantity != null || StrUtil.isNotBlank(practicalLocation) || StrUtil.isNotBlank(practicalRoomNo)) {
                BatchDetails batchDetails = batchDetailsService.getById(reagentConsumableInventoryFullVO.getBatchDetailsId());
                ReagentConsumableInventory reagentConsumableInventory = this.getById(batchDetails.getReagentConsumableInventoryId());
                ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getById(batchDetails.getReagentConsumableStashId());

                if (practicalQuantity != null) {//修改数量
                    reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() - batchDetails.getQuantity() + practicalQuantity);
                    batchDetails.setQuantity(practicalQuantity);
                    reagentConsumableStash.setTotalQuantity(practicalQuantity);
                    if (practicalQuantity == 0) {
                        batchDetails.setServiceStatus(-1);
                    }
                }
                if (StrUtil.isNotBlank(practicalRoomNo)) {
                    loop = true;//证明修改了位置，就要进行处理
                }
                if (StrUtil.isNotBlank(practicalLocation)) {
                    loop = true;
                }
                if (loop) {
//                    String location = practicalLocation;
//                    String substring = location.substring(location.length() - 4, location.length());
                    //修改位置，需清除原有位置信息，且需同步试剂耗材仓库库存信息表
                    R<CabinetCell> cabinetCellR = remoteCabinetService.queryCellByRoomNoAndLocation(practicalRoomNo, practicalLocation);
                    if (cabinetCellR == null) {
                        throw new RuntimeException(String.format("没有查询到你输入的房间与位置对应的柜子信息，是不是输错了呢？"));
                    }
                    CabinetCell cell = cabinetCellR.getData();
                    String cabinetId = cell.getCabinetId();
                    String cellName = baseMapper.getCellName(cabinetId);
                    String latticeId = cell.getId();
                    String boxId = cell.getCabinetId();
                    batchDetails.setRoomNo(practicalRoomNo);
                    reagentConsumableStash.setRoomNo(practicalRoomNo);
                    this.updateCabinet(batchDetails.getLatticeId(), batchDetails.getReagentConsumableInventoryId(), -1);//清除原来位置信息
                    batchDetails.setLatticeId(latticeId);
                    batchDetails.setBoxId(boxId);
                    String newLocation = "F" + batchDetails.getRoomNo().charAt(0) + "-" + batchDetails.getRoomNo() + "_" + cellName + "_" + practicalLocation;
                    batchDetails.setLocation(newLocation);
                    this.updateCabinet(latticeId, batchDetails.getReagentConsumableInventoryId(), 1);
                }
                reagentConsumableInventoryArrayList.add(reagentConsumableInventory);
                batchDetailsArrayList.add(batchDetails);
                reagentConsumableStashArrayList.add(reagentConsumableStash);
                long end = System.currentTimeMillis();

                System.out.println("消耗了" + (end - start) + "毫秒,修改了第" + index + "个试剂耗材");
                index++;
            }
        }
        if (this.updateBatchById(reagentConsumableInventoryArrayList) && batchDetailsService.updateBatchById(batchDetailsArrayList) && reagentConsumableStashService.updateBatchById(reagentConsumableStashArrayList)) {
            long e = System.currentTimeMillis();
            System.out.println("~~~~~~~~~~~~~~~~~消耗了" + (e - s) + "毫秒,共修改了" + index + "个试剂耗材");
            return true;
        }
        return false;
    }

}
