package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.ReagentConsumablesSetMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.ReagentConsumablesSetVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * (试剂耗材集合)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材集合) 服务实现类
 */
@SuppressWarnings("all")
@Service
public class ReagentConsumablesSetServiceImpl extends ServiceImpl<ReagentConsumablesSetMapper, ReagentConsumablesSet> implements ReagentConsumablesSetService {

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private ApplicationForUseService applicationForUseService;

    @Override
    public List<ReagentConsumablesSetVO> getList(String applicationForUseId) {

        ApplicationForUse applicationForUse = applicationForUseService.getById(applicationForUseId);

        LambdaQueryWrapper<ReagentConsumablesSet> reagentConsumablesSetLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumablesSetLambdaQueryWrapper.eq(ReagentConsumablesSet::getApplicationForUseId, applicationForUseId);

        List<ReagentConsumablesSet> list = this.list(reagentConsumablesSetLambdaQueryWrapper);

        List<ReagentConsumablesSetVO> reagentConsumablesSetVOS = new ArrayList<>();


        if (list.size() != 0) {
            //查找出试剂耗材名称
            for (ReagentConsumablesSet reagentConsumablesSet : list) {

                ReagentConsumablesSetVO reagentConsumablesSetVO = new ReagentConsumablesSetVO();

                BeanUtils.copyProperties(reagentConsumablesSet, reagentConsumablesSetVO);

                ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumablesSetVO.getReagentConsumableId());

                LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumablesSet.getReagentConsumableId());

                if (byId.getCategory().equals("试剂") | byId.getCategory().equals("耗材")) {

                    ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

                    reagentConsumablesSetVO.setTotalQuantity(one.getTotalQuantity());
                }

                //赋值领取码
                if (applicationForUse.getClaimCode() != null) {
                    reagentConsumablesSetVO.setClaimCode(applicationForUse.getClaimCode());
                }
                reagentConsumablesSetVO.setReagentConsumables(byId);

                if (reagentConsumablesSetVO.getReferenceMaterialId() != null) {
                    ReferenceMaterial referenceMaterial = referenceMaterialService.getById(reagentConsumablesSetVO.getReferenceMaterialId());
                    reagentConsumablesSetVO.setReferenceMaterialNumber(referenceMaterial.getNumber());
                    reagentConsumablesSetVO.setMass(referenceMaterial.getMass());
                    reagentConsumablesSetVO.setReferenceMaterialUnit(referenceMaterial.getReferenceMaterialUnit());

                }
                reagentConsumablesSetVOS.add(reagentConsumablesSetVO);
            }
        }
        return reagentConsumablesSetVOS;
    }
}
