package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.StandardReserveSolutionDTO;
import digital.laboratory.platform.reagent.dto.StandardReserveSolutionFullDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.StandardReserveSolutionMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import digital.laboratory.platform.sys.feign.RemoteCabinetService;
import feign.Response;
//import io.seata.spring.annotation.GlobalTransactional;
import io.seata.spring.annotation.GlobalTransactional;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * (标准储备溶液配制及使用记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准储备溶液配制及使用记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class StandardReserveSolutionServiceImpl extends ServiceImpl<StandardReserveSolutionMapper, StandardReserveSolution> implements StandardReserveSolutionService {

    @Autowired
    private SolutionUseFormService solutionUseFormService;

    @Autowired
    private RemoteCabinetService remoteCabinetService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private BatchDetailsService batchDetailsService;


    @Override//通过ID查询标准溶液的配置记录
    public StandardReserveSolutionVO getStandardReserveSolutionVOById(String id) {

        StandardReserveSolutionVO standardReserveSolutionVOById = baseMapper.getStandardReserveSolutionVOById(id);

        standardReserveSolutionVOById.setPrintName(standardReserveSolutionVOById.getSolutionName());

        String format = String.format("%.4f", standardReserveSolutionVOById.getConfigurationConcentration());
        standardReserveSolutionVOById.setConfigurationDatePrint(standardReserveSolutionVOById.getDispenserName() + "-" + LocalDateTimeUtil.format(standardReserveSolutionVOById.getConfigurationDate(), "yyyy-MM-dd") + "-" + format);
        //查出该标准储备溶液对应的所有使用记录
        List<SolutionUseFormVO> solutionUseFormVOList = solutionUseFormService.getSolutionUseFormVOList(standardReserveSolutionVOById.getId());

        if (solutionUseFormVOList != null) {
            standardReserveSolutionVOById.setSolutionUseFormVOList(solutionUseFormVOList);
        }
        return standardReserveSolutionVOById;
    }

    @Override//配置标准储备溶液
    @Transactional
    public StandardReserveSolutionVO addById(StandardReserveSolutionDTO standardReserveSolutionDTO, DLPUser dlpUser) {

        if (referenceMaterialService.getById(standardReserveSolutionDTO.getReferenceMaterialId()).getStatus() != -3) {

            throw new RuntimeException(String.format("请先领用标准物质后，再进行操作"));
        }


        StandardReserveSolution standardReserveSolution = new StandardReserveSolution();

        BeanUtils.copyProperties(standardReserveSolutionDTO, standardReserveSolution);

        standardReserveSolution.setReferenceMaterialScale(standardReserveSolutionDTO.getReferenceMaterialScale());
        standardReserveSolution.setConstantVolume(standardReserveSolutionDTO.getConstantVolume());

        standardReserveSolution.setId(IdWorker.get32UUID().toUpperCase());

        standardReserveSolution.setDispenserId(dlpUser.getId());

        standardReserveSolution.setConfigurationDate(LocalDateTime.now());

        standardReserveSolution.setStatus(0);
        //计算浓度
        Double v = standardReserveSolution.getReferenceMaterialScale() / standardReserveSolution.getConstantVolume();

        standardReserveSolution.setConfigurationConcentration(v);

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableName, standardReserveSolution.getSolutionName());
        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getConfigurationConcentration, standardReserveSolution.getConfigurationConcentration());

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);
        //判断是否存在过该标准储备溶液，若未存在，则将其存入试剂耗材类
        if (one == null) {

            ReferenceMaterial byId = referenceMaterialService.getById(standardReserveSolution.getReferenceMaterialId());
            //创建一个标准储备溶液的种类对象
            ReagentConsumables reagentConsumables1 = reagentConsumablesService.getById(byId.getReagentConsumableId());

            reagentConsumables1.setReagentConsumableId(IdWorker.get32UUID().toUpperCase());
            reagentConsumables1.setReagentConsumableName(standardReserveSolution.getSolutionName());
            reagentConsumables1.setConfigurationConcentration(standardReserveSolution.getConfigurationConcentration());
            reagentConsumables1.setCategory("标准储备溶液");
            reagentConsumables1.setCreateTime(LocalDateTime.now());
            reagentConsumables1.setUpdateTime(LocalDateTime.now());

            reagentConsumablesService.save(reagentConsumables1);
            //创建标准储备溶液的库存对象
            ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();

            BeanUtils.copyProperties(reagentConsumables1, reagentConsumableInventory);

            reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());

            reagentConsumableInventory.setStatus(1);

            reagentConsumableInventory.setConfigurationConcentration(standardReserveSolution.getConfigurationConcentration());
            reagentConsumableInventory.setCreateTime(LocalDateTime.now());
            reagentConsumableInventory.setUpdateTime(LocalDateTime.now());
            reagentConsumableInventory.setTotalQuantity(0);

            //创建标准储备溶液对象
            ReferenceMaterial referenceMaterial = new ReferenceMaterial();
            referenceMaterial.setMass(standardReserveSolutionDTO.getConstantVolume());

            ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(standardReserveSolutionDTO.getReferenceMaterialId());

            String prefix = referenceMaterialServiceById.getNumber() + "-";

            List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                    .likeRight("number", prefix)
                    .orderByDesc("number"));

            int newNo = 1;

            if ((list != null) && (list.size() > 0)) {
                ReferenceMaterial referenceMaterial1 = list.get(0);
                String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                try {
                    int maxno = Integer.parseUnsignedInt(strMaxNo);
                    newNo = maxno + 1;
                } catch (NumberFormatException e) {
                    // 如果后缀有非数字, 则无视之, 重头编码
                    newNo = 1;
                }
            }
            //创建标准溶液对象，并生成编号
            referenceMaterial.setStatus(-5);
            referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
            referenceMaterial.setReagentConsumableId(reagentConsumableInventory.getReagentConsumableId());
            referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
            referenceMaterial.setNumber(prefix + newNo);
            referenceMaterial.setReagentConsumableName(reagentConsumableInventory.getReagentConsumableName());
            referenceMaterial.setReferenceMaterialUnit("ml");
            referenceMaterial.setMass(standardReserveSolutionDTO.getConstantVolume());

            ReferenceMaterial byId1 = referenceMaterialService.getById(standardReserveSolution.getReferenceMaterialId());

            BatchDetails batchDetails = batchDetailsService.getById(byId.getBatchDetailsId());

            referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());

            referenceMaterialService.save(referenceMaterial);

            standardReserveSolution.setReferenceId(referenceMaterial.getId());


            standardReserveSolution.setSolutionNumbering(referenceMaterial.getNumber());

            if (reagentConsumableInventoryService.save(reagentConsumableInventory) && this
                    .save(standardReserveSolution)) {
                StandardReserveSolutionFullVO byFullVOId = this.getByFullVOId(standardReserveSolution.getId());
                return byFullVOId;
            } else {
                throw new RuntimeException(String.format("配置失败"));
            }

        } else {

            ReferenceMaterial byId1 = referenceMaterialService.getById(standardReserveSolution.getReferenceMaterialId());

            BatchDetails batchDetails = batchDetailsService.getById(byId1.getBatchDetailsId());

            ReferenceMaterial referenceMaterial = new ReferenceMaterial();

            LocalDate date = LocalDate.now(); // get the current date
            //获取当前年月日
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

            String format = date.format(formatter);

            //创建标准溶液对象，并生成编号
            referenceMaterial.setStatus(-5);
            referenceMaterial.setMass(standardReserveSolutionDTO.getConstantVolume());
            referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
            referenceMaterial.setReagentConsumableId(one.getReagentConsumableId());
            referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
            referenceMaterial.setReferenceMaterialUnit("ml");
            referenceMaterial.setMass(standardReserveSolutionDTO.getConstantVolume());

            Calendar calendar = Calendar.getInstance();

            ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(standardReserveSolutionDTO.getReferenceMaterialId());

            String prefix = referenceMaterialServiceById.getNumber() + "-";

            List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                    .likeRight("number", prefix)
                    .orderByDesc("number"));

            int newNo = 1;

            if ((list != null) && (list.size() > 0)) {
                ReferenceMaterial referenceMaterial1 = list.get(0);
                String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                try {
                    int maxno = Integer.parseUnsignedInt(strMaxNo);
                    newNo = maxno + 1;
                } catch (NumberFormatException e) {
                    // 如果后缀有非数字, 则无视之, 重头编码
                    newNo = 1;
                }
            }

            referenceMaterial.setNumber(prefix + newNo);

            referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());

            standardReserveSolution.setReferenceId(referenceMaterial.getId());

            referenceMaterialService.save(referenceMaterial);
            //将存储信息录入仓库表

            standardReserveSolution.setSolutionNumbering(referenceMaterial.getNumber());

            if (this.save(standardReserveSolution)) {

                StandardReserveSolutionVO standardReserveSolutionVOById = this.getStandardReserveSolutionVOById(standardReserveSolution.getId());

                return standardReserveSolutionVOById;
            } else throw new RuntimeException(String.format("配置失败"));

        }

    }

    @Override//标准储备溶液入库任务
    @Transactional
    public IPage<StandardReserveSolutionVO> getStandardReserveSolutionVOList(IPage<StandardReserveSolution> page, QueryWrapper<StandardReserveSolution> qw) {

        IPage<StandardReserveSolutionVO> standardReserveSolutionVOPage = baseMapper.getStandardReserveSolutionVOPage(page, qw);

        List<StandardReserveSolutionVO> records = standardReserveSolutionVOPage.getRecords();

        for (StandardReserveSolutionVO record : records) {

            ReferenceMaterial byId = referenceMaterialService.getById(record.getReferenceId());

            if (byId != null) {
                record.setLocation(byId.getLocation());
                record.setLatticeId(byId.getLatticeId());
                record.setMass(byId.getMass());
                record.setReferenceMaterialUnit(byId.getReferenceMaterialUnit());
            }
        }
        return standardReserveSolutionVOPage;
    }

    @Override//标准储备溶液配置入库
    @GlobalTransactional
    public StandardReserveSolutionFullVO warehousingById(StandardReserveSolutionFullDTO standardReserveSolutionFullDTO) {

        String id = standardReserveSolutionFullDTO.getId();

        String location = standardReserveSolutionFullDTO.getLocation();
        String latticeId = standardReserveSolutionFullDTO.getLatticeId();
        String boxId = standardReserveSolutionFullDTO.getBoxId();

        String warehousingRemarks = standardReserveSolutionFullDTO.getWarehousingRemarks();


        //提供格子位置与标准储备溶液配置表ID
        StandardReserveSolution byId = this.getById(id);
        byId.setStatus(1);

        byId.setWarehousingRemarks(warehousingRemarks);
        //为标准储备溶液对象赋值位置信息
        ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(byId.getReferenceId());

        referenceMaterialServiceById.setBoxId(boxId);
        referenceMaterialServiceById.setLocation(location);
        referenceMaterialServiceById.setLatticeId(latticeId);
        referenceMaterialServiceById.setStatus(0);

        referenceMaterialService.updateById(referenceMaterialServiceById);

        ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getById(referenceMaterialServiceById.getReagentConsumableInventoryId());
        reagentConsumableInventoryService.updateCabinet(referenceMaterialServiceById.getLatticeId(), referenceMaterialServiceById.getId(), 1);
        if (reagentConsumableInventory.getTotalQuantity() != null) {

            reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() + 1);

        } else {
            reagentConsumableInventory.setTotalQuantity(1);
        }

        if (this.updateById(byId) && reagentConsumableInventoryService.updateById(reagentConsumableInventory)) {

            StandardReserveSolutionFullVO byFullVOId = this.getByFullVOId(byId.getId());
            return byFullVOId;
        } else {
            throw new RuntimeException(String.format("入库失败"));
        }
    }

    @Override//通过ID查找已入库的标准储备溶液信息
    public StandardReserveSolutionFullVO getByFullVOId(String id) {

        StandardReserveSolutionVO standardReserveSolutionVOById = this.getStandardReserveSolutionVOById(id);

        StandardReserveSolutionFullVO standardReserveSolutionFullVO = new StandardReserveSolutionFullVO();

        BeanUtils.copyProperties(standardReserveSolutionVOById, standardReserveSolutionFullVO);

        return standardReserveSolutionFullVO;
    }

    @Override
    public void standardReserveSolutionTablePDF(StandardReserveSolutionVO standardReserveSolutionVO, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("standardReserveSolutionTablePDF.................");

        String id = standardReserveSolutionVO.getId();

        List<SolutionUseFormVO> voList = standardReserveSolutionVO.getSolutionUseFormVOList();

        voList.sort((a, b) -> a.getCreateTime().compareTo(b.getCreateTime()));

        HashMap solutionUseFormVOHashMap = new HashMap<String, Object>();
        List<Map<String, Object>> maps = new ArrayList<>();

        for (int i = 0; i < voList.size(); i++) {
            voList.get(i).setFormat(LocalDateTimeUtil.format(voList.get(i).getCreateTime(), "yyyy年MM月dd日"));
            solutionUseFormVOHashMap.put("userName" + (i + 1), voList.get(i).getUserName());
            solutionUseFormVOHashMap.put("format" + (i + 1), voList.get(i).getFormat());
            solutionUseFormVOHashMap.put("quantityUsed" + (i + 1), voList.get(i).getQuantityUsed());
        }
        maps.add(solutionUseFormVOHashMap);


//        if (voList.size()<7){
//            int s = 7-voList.size();
//            for (int i = 0; i < s; i++) {
//                SolutionUseFormVO solutionUseFormVO = new SolutionUseFormVO();
//                solutionUseFormVO.setOrderOfUse(voList.size()+1);
//                voList.add(solutionUseFormVO);
//            }
//        }else {
//            int s = 7-voList.size()%7;
//            for (int i = 0; i < s; i++) {
//                SolutionUseFormVO solutionUseFormVO = new SolutionUseFormVO();
//                voList.add(solutionUseFormVO);
//            }
//        }


        String applyFileName = "标准储备溶液配制及使用记录表-" + id;
        //-----------------------------
        // 生成 word 版本的 验收记录表

        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        ossFile.fileGet("template" + "/" + "标准储备溶液配制及使用记录表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);

        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        builder.bind("maps", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("maps", maps);
                    put("year", year);
                    put("solutionName", standardReserveSolutionVO.getSolutionName());
                    put("solutionNumbering", standardReserveSolutionVO.getSolutionNumbering());
                    put("referenceMaterialNumber", standardReserveSolutionVO.getReferenceMaterialNumber());
                    put("referenceMaterialScale", standardReserveSolutionVO.getReferenceMaterialScale().toString());
                    put("useOfSolvent", standardReserveSolutionVO.getUseOfSolvent());
                    put("constantVolume", standardReserveSolutionVO.getConstantVolume());
                    put("configurationConcentration", String.format("%.4f", standardReserveSolutionVO.getConfigurationConcentration()));
                    put("dispenserName", standardReserveSolutionVO.getDispenserName());
                    put("validityPeriod", standardReserveSolutionVO.getValidityPeriod());
                    put("remarks", standardReserveSolutionVO.getRemarks());
                    put("configurationDate", LocalDateTimeUtil.format(standardReserveSolutionVO.getConfigurationDate(), "yyyy年MM月dd日"));
                    put("referenceMaterialUnit", referenceMaterialService.getById(standardReserveSolutionVO.getReferenceMaterialId()).getReferenceMaterialUnit());
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();

        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "standardReserveSolution" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();
        System.out.println(String.format("转换为 PDF 结束"));
    }
}
